@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 2016 Free Software Foundation, Inc.
+ * Copyright (c) 2016-2021 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -20,13 +20,13 @@
  * Libwget version information
  */
 
-#ifndef _LIBWGET_WGETVER_H
-#define _LIBWGET_WGETVER_H
+#ifndef WGET_WGETVER_H
+#define WGET_WGETVER_H
 
-#define LIBWGET_VERSION "0.0.1"
-#define LIBWGET_VERSION_MAJOR 0
+#define LIBWGET_VERSION "2.0.0"
+#define LIBWGET_VERSION_MAJOR 2
 #define LIBWGET_VERSION_MINOR 0
-#define LIBWGET_VERSION_PATCH 1
-#define LIBWGET_VERSION_NUMBER 0x000001
+#define LIBWGET_VERSION_PATCH 0
+#define LIBWGET_VERSION_NUMBER 0x020000
 
-#endif /* _LIBWGET_WGETVER_H */
+#endif /* WGET_WGETVER_H */
