@@ -1,10 +1,10 @@
 /* Creation of subprocesses, communicating via pipes.
-   Copyright (C) 2001-2004, 2006-2024 Free Software Foundation, Inc.
+   Copyright (C) 2001-2004, 2006-2021 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation, either version 3 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -33,11 +33,10 @@
 #include <unistd.h>
 
 #include "canonicalize.h"
-#include <error.h>
+#include "error.h"
 #include "fatal-signal.h"
 #include "filename.h"
 #include "findprog.h"
-#include "windows-path.h"
 #include "unistd-safer.h"
 #include "wait-process.h"
 #include "xalloc.h"
@@ -139,7 +138,6 @@ static pid_t
 create_pipe (const char *progname,
              const char *prog_path,
              const char * const *prog_argv,
-             const char * const *dll_dirs,
              const char *directory,
              bool pipe_stdin, bool pipe_stdout,
              const char *prog_stdin, const char *prog_stdout,
@@ -182,8 +180,9 @@ create_pipe (const char *progname,
                 canonicalize_filename_mode (prog_path, CAN_MISSING | CAN_NOLINKS);
               if (absolute_prog == NULL)
                 {
+                  saved_errno = errno;
                   free (prog_path_to_free);
-                  goto fail_with_errno;
+                  goto fail_with_saved_errno;
                 }
               free (prog_path_to_free);
               prog_path_to_free = absolute_prog;
@@ -195,24 +194,25 @@ create_pipe (const char *progname,
         }
     }
 
+#if ((defined _WIN32 && !defined __CYGWIN__) && SPAWN_PIPE_IMPL_AVOID_POSIX_SPAWN) || defined __KLIBC__
+
+  /* Native Windows API.
+     This uses _pipe(), dup2(), and _spawnv().  It could also be implemented
+     using the low-level functions CreatePipe(), DuplicateHandle(),
+     CreateProcess() and _open_osfhandle(); see the GNU make and GNU clisp
+     and cvs source code.  */
+  char *argv_mem_to_free;
   int ifd[2];
   int ofd[2];
+  int child;
+  int nulloutfd;
+  int stdinfd;
+  int stdoutfd;
+
+  const char **argv = prepare_spawn (prog_argv, &argv_mem_to_free);
+  if (argv == NULL)
+    xalloc_die ();
 
-  /* It is important to create the file descriptors with the close-on-exec bit
-     set.
-     * In the child process that we are about to create here, the file
-       descriptors ofd[0] -> STDIN_FILENO and ifd[1] -> STDOUT_FILENO will be
-       preserved across exec, because each dup2 call scheduled by
-       posix_spawn_file_actions_adddup2 creates a file descriptor with the
-       close-on-exec bit clear.  Similarly on native Windows, where we use
-       explicit DuplicateHandle calls, and on kLIBC, where we use explicit dup2
-       calls.
-     * In the parent process, we close ofd[0] and ifd[1]; so, ofd[1] and ofd[0]
-       are still open. But if the parent process spawns another child process
-       later, if ofd[1] and ofd[0] were inherited by that child process, the
-       "end of input" / "end of output" detection would not work any more.  The
-       parent or the child process would block, as long as that other child
-       process is running.  */
   if (pipe_stdout)
     if (pipe2_safer (ifd, O_BINARY | O_CLOEXEC) < 0)
       error (EXIT_FAILURE, errno, _("cannot create pipe"));
@@ -228,23 +228,6 @@ create_pipe (const char *progname,
  *
  */
 
-#if ((defined _WIN32 && !defined __CYGWIN__) && SPAWN_PIPE_IMPL_AVOID_POSIX_SPAWN) || defined __KLIBC__
-
-  /* Native Windows API.
-     This uses _pipe(), dup2(), and _spawnv().  It could also be implemented
-     using the low-level functions CreatePipe(), DuplicateHandle(),
-     CreateProcess() and _open_osfhandle(); see the GNU make and GNU clisp
-     and cvs source code.  */
-  char *argv_mem_to_free;
-  int child;
-  int nulloutfd;
-  int stdinfd;
-  int stdoutfd;
-
-  const char **argv = prepare_spawn (prog_argv, &argv_mem_to_free);
-  if (argv == NULL)
-    xalloc_die ();
-
   child = -1;
 
 # if (defined _WIN32 && !defined __CYGWIN__) && SPAWN_PIPE_IMPL_AVOID_POSIX_SPAWN
@@ -315,8 +298,7 @@ create_pipe (const char *progname,
         (HANDLE) _get_osfhandle (null_stderr ? nulloutfd : STDERR_FILENO);
 
       child = spawnpvech (P_NOWAIT, prog_path, argv + 1,
-                          (const char * const *) environ, dll_dirs,
-                          directory,
+                          (const char * const *) environ, directory,
                           stdin_handle, stdout_handle, stderr_handle);
 #  if 0 /* Executing arbitrary files as shell scripts is unsecure.  */
       if (child == -1 && errno == ENOEXEC)
@@ -326,8 +308,7 @@ create_pipe (const char *progname,
              a hidden element "sh.exe" to argv.  */
           argv[1] = prog_path;
           child = spawnpvech (P_NOWAIT, argv[0], argv,
-                              (const char * const *) environ, dll_dirs,
-                              directory,
+                              (const char * const *) environ, directory,
                               stdin_handle, stdout_handle, stderr_handle);
         }
 #  endif
@@ -360,43 +341,62 @@ create_pipe (const char *progname,
     }
 
 # else /* __KLIBC__ */
-  int new_stdin = STDIN_FILENO;
-  int new_stdout = STDOUT_FILENO;
-  int new_stderr = STDERR_FILENO;
+  if (!(directory == NULL || strcmp (directory, ".") == 0))
+    {
+      /* A directory argument is not supported in this implementation.  */
+      saved_errno = EINVAL;
+      goto fail_with_saved_errno;
+    }
+
+  int orig_stdin;
+  int orig_stdout;
+  int orig_stderr;
+
+  /* Save standard file handles of parent process.  */
+  if (pipe_stdin || prog_stdin != NULL)
+    orig_stdin = dup_safer_noinherit (STDIN_FILENO);
+  if (pipe_stdout || prog_stdout != NULL)
+    orig_stdout = dup_safer_noinherit (STDOUT_FILENO);
+  if (null_stderr)
+    orig_stderr = dup_safer_noinherit (STDERR_FILENO);
 
   /* Create standard file handles of child process.  */
   nulloutfd = -1;
   stdinfd = -1;
   stdoutfd = -1;
-  if ((!pipe_stdin || (new_stdin = ofd[0]) >= 0)
-      && (!pipe_stdout || (new_stdout = ifd[1]) >= 0)
+  if ((!pipe_stdin || dup2 (ofd[0], STDIN_FILENO) >= 0)
+      && (!pipe_stdout || dup2 (ifd[1], STDOUT_FILENO) >= 0)
       && (!null_stderr
-          || ((nulloutfd = open ("NUL", O_RDWR | O_CLOEXEC, 0)) >= 0
-              && (new_stderr = nulloutfd) >= 0))
+          || ((nulloutfd = open ("NUL", O_RDWR, 0)) >= 0
+              && (nulloutfd == STDERR_FILENO
+                  || (dup2 (nulloutfd, STDERR_FILENO) >= 0
+                      && close (nulloutfd) >= 0))))
       && (pipe_stdin
           || prog_stdin == NULL
-          || ((stdinfd = open (prog_stdin, O_RDONLY | O_CLOEXEC, 0)) >= 0
-              && (new_stdin = stdinfd) >= 0))
+          || ((stdinfd = open (prog_stdin, O_RDONLY, 0)) >= 0
+              && (stdinfd == STDIN_FILENO
+                  || (dup2 (stdinfd, STDIN_FILENO) >= 0
+                      && close (stdinfd) >= 0))))
       && (pipe_stdout
           || prog_stdout == NULL
-          || ((stdoutfd = open (prog_stdout, O_WRONLY | O_CLOEXEC, 0)) >= 0
-              && (new_stdout = stdoutfd) >= 0)))
+          || ((stdoutfd = open (prog_stdout, O_WRONLY, 0)) >= 0
+              && (stdoutfd == STDOUT_FILENO
+                  || (dup2 (stdoutfd, STDOUT_FILENO) >= 0
+                      && close (stdoutfd) >= 0)))))
     /* The child process doesn't inherit ifd[0], ifd[1], ofd[0], ofd[1],
        but it inherits all open()ed or dup2()ed file handles (which is what
        we want in the case of STD*_FILENO).  */
     {
-      child = spawnpvech (P_NOWAIT, prog_path, argv + 1,
-                          (const char **) environ, directory,
-                          new_stdin, new_stdout, new_stderr);
+      child = _spawnvpe (P_NOWAIT, prog_path, argv + 1,
+                         (const char **) environ);
 #  if 0 /* Executing arbitrary files as shell scripts is unsecure.  */
       if (child == -1 && errno == ENOEXEC)
         {
           /* prog is not a native executable.  Try to execute it as a
              shell script.  Note that prepare_spawn() has already prepended
              a hidden element "sh.exe" to argv.  */
-          child = spawnpvech (P_NOWAIT, argv[0], argv,
-                              (const char **) environ, directory,
-                              new_stdin, new_stdout, new_stderr);
+          child = _spawnvpe (P_NOWAIT, argv[0], argv,
+                             (const char **) environ);
         }
 #  endif
     }
@@ -409,6 +409,14 @@ create_pipe (const char *progname,
   if (nulloutfd >= 0)
     close (nulloutfd);
 
+  /* Restore standard file handles of parent process.  */
+  if (null_stderr)
+    undup_safer_noinherit (orig_stderr, STDERR_FILENO);
+  if (pipe_stdout || prog_stdout != NULL)
+    undup_safer_noinherit (orig_stdout, STDOUT_FILENO);
+  if (pipe_stdin || prog_stdin != NULL)
+    undup_safer_noinherit (orig_stdin, STDIN_FILENO);
+
   if (pipe_stdin)
     close (ofd[0]);
   if (pipe_stdout)
@@ -437,8 +445,8 @@ create_pipe (const char *progname,
 #else
 
   /* Unix API.  */
-  char **child_environ;
-  char **malloced_environ;
+  int ifd[2];
+  int ofd[2];
   sigset_t blocked_signals;
   posix_spawn_file_actions_t actions;
   bool actions_allocated;
@@ -447,17 +455,20 @@ create_pipe (const char *progname,
   int err;
   pid_t child;
 
-  child_environ = environ;
-  malloced_environ = NULL;
-# if defined _WIN32 || defined __CYGWIN__
-  if (dll_dirs != NULL && dll_dirs[0] != NULL)
-    {
-      malloced_environ = extended_environ (dll_dirs);
-      if (malloced_environ == NULL)
-        goto fail_with_errno;
-      child_environ = malloced_environ;
-    }
-# endif
+  if (pipe_stdout)
+    if (pipe_safer (ifd) < 0)
+      error (EXIT_FAILURE, errno, _("cannot create pipe"));
+  if (pipe_stdin)
+    if (pipe_safer (ofd) < 0)
+      error (EXIT_FAILURE, errno, _("cannot create pipe"));
+/* Data flow diagram:
+ *
+ *           write        system         read
+ *    parent  ->   ofd[1]   ->   ofd[0]   ->   child       if pipe_stdin
+ *    parent  <-   ifd[0]   <-   ifd[1]   <-   child       if pipe_stdout
+ *           read         system         write
+ *
+ */
 
   if (slave_process)
     {
@@ -517,26 +528,24 @@ create_pipe (const char *progname,
 # if defined _WIN32 && !defined __CYGWIN__
                       (err = posix_spawnattr_setpgroup (&attrs, 0)) != 0
                       || (err = posix_spawnattr_setflags (&attrs,
-                                                          POSIX_SPAWN_SETPGROUP))
+                                                         POSIX_SPAWN_SETPGROUP))
                          != 0
 # else
                       (err = posix_spawnattr_setsigmask (&attrs,
                                                          &blocked_signals))
                       != 0
                       || (err = posix_spawnattr_setflags (&attrs,
-                                                          POSIX_SPAWN_SETSIGMASK))
+                                                        POSIX_SPAWN_SETSIGMASK))
                          != 0
 # endif
              )   )   )
           || (err = (directory != NULL
                      ? posix_spawn (&child, prog_path, &actions,
                                     attrs_allocated ? &attrs : NULL,
-                                    (char * const *) prog_argv,
-                                    child_environ)
+                                    (char * const *) prog_argv, environ)
                      : posix_spawnp (&child, prog_path, &actions,
                                      attrs_allocated ? &attrs : NULL,
-                                     (char * const *) prog_argv,
-                                     child_environ)))
+                                     (char * const *) prog_argv, environ)))
              != 0))
     {
       if (actions_allocated)
@@ -545,11 +554,6 @@ create_pipe (const char *progname,
         posix_spawnattr_destroy (&attrs);
       if (slave_process)
         unblock_fatal_signals ();
-      if (malloced_environ != NULL)
-        {
-          free (malloced_environ[0]);
-          free (malloced_environ);
-        }
       if (pipe_stdout)
         {
           close (ifd[0]);
@@ -572,11 +576,6 @@ create_pipe (const char *progname,
       register_slave_subprocess (child);
       unblock_fatal_signals ();
     }
-  if (malloced_environ != NULL)
-    {
-      free (malloced_environ[0]);
-      free (malloced_environ);
-    }
   if (pipe_stdin)
     close (ofd[0]);
   if (pipe_stdout)
@@ -612,14 +611,12 @@ create_pipe (const char *progname,
 pid_t
 create_pipe_bidi (const char *progname,
                   const char *prog_path, const char * const *prog_argv,
-                  const char * const *dll_dirs,
                   const char *directory,
                   bool null_stderr,
                   bool slave_process, bool exit_on_error,
                   int fd[2])
 {
-  pid_t result = create_pipe (progname, prog_path, prog_argv, dll_dirs,
-                              directory,
+  pid_t result = create_pipe (progname, prog_path, prog_argv, directory,
                               true, true, NULL, NULL,
                               null_stderr, slave_process, exit_on_error,
                               fd);
@@ -636,15 +633,13 @@ create_pipe_bidi (const char *progname,
 pid_t
 create_pipe_in (const char *progname,
                 const char *prog_path, const char * const *prog_argv,
-                const char * const *dll_dirs,
                 const char *directory,
                 const char *prog_stdin, bool null_stderr,
                 bool slave_process, bool exit_on_error,
                 int fd[1])
 {
   int iofd[2];
-  pid_t result = create_pipe (progname, prog_path, prog_argv, dll_dirs,
-                              directory,
+  pid_t result = create_pipe (progname, prog_path, prog_argv, directory,
                               false, true, prog_stdin, NULL,
                               null_stderr, slave_process, exit_on_error,
                               iofd);
@@ -663,15 +658,13 @@ create_pipe_in (const char *progname,
 pid_t
 create_pipe_out (const char *progname,
                  const char *prog_path, const char * const *prog_argv,
-                 const char * const *dll_dirs,
                  const char *directory,
                  const char *prog_stdout, bool null_stderr,
                  bool slave_process, bool exit_on_error,
                  int fd[1])
 {
   int iofd[2];
-  pid_t result = create_pipe (progname, prog_path, prog_argv, dll_dirs,
-                              directory,
+  pid_t result = create_pipe (progname, prog_path, prog_argv, directory,
                               true, false, NULL, prog_stdout,
                               null_stderr, slave_process, exit_on_error,
                               iofd);
