@@ -1,6 +1,6 @@
 /*
- * Copyright(c) 2012 Tim Ruehsen
- * Copyright(c) 2015-2019 Free Software Foundation, Inc.
+ * Copyright (c) 2012 Tim Ruehsen
+ * Copyright (c) 2015-2021 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -110,15 +110,15 @@ bool wget_iri_supported(const wget_iri *iri)
 
 /* \cond _hide_internal_symbols */
 #define IRI_CTYPE_GENDELIM (1<<0)
-#define _iri_isgendelim(c) (iri_ctype[(unsigned char)(c)] & IRI_CTYPE_GENDELIM)
+#define iri_isgendelim(c) (iri_ctype[(unsigned char)(c)] & IRI_CTYPE_GENDELIM)
 
 #define IRI_CTYPE_SUBDELIM (1<<1)
-#define _iri_issubdelim(c) (iri_ctype[(unsigned char)(c)] & IRI_CTYPE_SUBDELIM)
+#define iri_issubdelim(c) (iri_ctype[(unsigned char)(c)] & IRI_CTYPE_SUBDELIM)
 
 #define IRI_CTYPE_UNRESERVED (1<<2)
-#define _iri_isunreserved(c) (iri_ctype[(unsigned char)(c)] & IRI_CTYPE_UNRESERVED)
+#define iri_isunreserved(c) (iri_ctype[(unsigned char)(c)] & IRI_CTYPE_UNRESERVED)
 
-#define _iri_isscheme(c) (c_isalnum(c) || c == '+' || c == '-' || c == '.')
+#define iri_isscheme(c) (c_isalnum(c) || c == '+' || c == '-' || c == '.')
 /* \endcond */
 
 static const unsigned char
@@ -134,7 +134,6 @@ static const unsigned char
 		['!'] = IRI_CTYPE_SUBDELIM,
 		['$'] = IRI_CTYPE_SUBDELIM,
 		['&'] = IRI_CTYPE_SUBDELIM,
-//		['\\'] = IRI_CTYPE_SUBDELIM,
 		['\''] = IRI_CTYPE_SUBDELIM,
 		['('] = IRI_CTYPE_SUBDELIM,
 		[')'] = IRI_CTYPE_SUBDELIM,
@@ -144,6 +143,68 @@ static const unsigned char
 		[';'] = IRI_CTYPE_SUBDELIM,
 		['='] = IRI_CTYPE_SUBDELIM,
 
+		['0'] = IRI_CTYPE_UNRESERVED,
+		['1'] = IRI_CTYPE_UNRESERVED,
+		['2'] = IRI_CTYPE_UNRESERVED,
+		['3'] = IRI_CTYPE_UNRESERVED,
+		['4'] = IRI_CTYPE_UNRESERVED,
+		['5'] = IRI_CTYPE_UNRESERVED,
+		['6'] = IRI_CTYPE_UNRESERVED,
+		['7'] = IRI_CTYPE_UNRESERVED,
+		['8'] = IRI_CTYPE_UNRESERVED,
+		['9'] = IRI_CTYPE_UNRESERVED,
+		['a'] = IRI_CTYPE_UNRESERVED,
+		['b'] = IRI_CTYPE_UNRESERVED,
+		['c'] = IRI_CTYPE_UNRESERVED,
+		['d'] = IRI_CTYPE_UNRESERVED,
+		['e'] = IRI_CTYPE_UNRESERVED,
+		['f'] = IRI_CTYPE_UNRESERVED,
+		['g'] = IRI_CTYPE_UNRESERVED,
+		['h'] = IRI_CTYPE_UNRESERVED,
+		['i'] = IRI_CTYPE_UNRESERVED,
+		['j'] = IRI_CTYPE_UNRESERVED,
+		['k'] = IRI_CTYPE_UNRESERVED,
+		['l'] = IRI_CTYPE_UNRESERVED,
+		['m'] = IRI_CTYPE_UNRESERVED,
+		['n'] = IRI_CTYPE_UNRESERVED,
+		['o'] = IRI_CTYPE_UNRESERVED,
+		['p'] = IRI_CTYPE_UNRESERVED,
+		['q'] = IRI_CTYPE_UNRESERVED,
+		['r'] = IRI_CTYPE_UNRESERVED,
+		['s'] = IRI_CTYPE_UNRESERVED,
+		['t'] = IRI_CTYPE_UNRESERVED,
+		['u'] = IRI_CTYPE_UNRESERVED,
+		['v'] = IRI_CTYPE_UNRESERVED,
+		['w'] = IRI_CTYPE_UNRESERVED,
+		['x'] = IRI_CTYPE_UNRESERVED,
+		['y'] = IRI_CTYPE_UNRESERVED,
+		['z'] = IRI_CTYPE_UNRESERVED,
+		['A'] = IRI_CTYPE_UNRESERVED,
+		['B'] = IRI_CTYPE_UNRESERVED,
+		['C'] = IRI_CTYPE_UNRESERVED,
+		['D'] = IRI_CTYPE_UNRESERVED,
+		['E'] = IRI_CTYPE_UNRESERVED,
+		['F'] = IRI_CTYPE_UNRESERVED,
+		['G'] = IRI_CTYPE_UNRESERVED,
+		['H'] = IRI_CTYPE_UNRESERVED,
+		['I'] = IRI_CTYPE_UNRESERVED,
+		['J'] = IRI_CTYPE_UNRESERVED,
+		['K'] = IRI_CTYPE_UNRESERVED,
+		['L'] = IRI_CTYPE_UNRESERVED,
+		['M'] = IRI_CTYPE_UNRESERVED,
+		['N'] = IRI_CTYPE_UNRESERVED,
+		['O'] = IRI_CTYPE_UNRESERVED,
+		['P'] = IRI_CTYPE_UNRESERVED,
+		['Q'] = IRI_CTYPE_UNRESERVED,
+		['R'] = IRI_CTYPE_UNRESERVED,
+		['S'] = IRI_CTYPE_UNRESERVED,
+		['T'] = IRI_CTYPE_UNRESERVED,
+		['U'] = IRI_CTYPE_UNRESERVED,
+		['V'] = IRI_CTYPE_UNRESERVED,
+		['W'] = IRI_CTYPE_UNRESERVED,
+		['X'] = IRI_CTYPE_UNRESERVED,
+		['Y'] = IRI_CTYPE_UNRESERVED,
+		['Z'] = IRI_CTYPE_UNRESERVED,
 		['-'] = IRI_CTYPE_UNRESERVED,
 		['.'] = IRI_CTYPE_UNRESERVED,
 		['_'] = IRI_CTYPE_UNRESERVED,
@@ -160,7 +221,7 @@ static const unsigned char
 bool wget_iri_isgendelim(char c)
 {
 	// return strchr(":/?#[]@",c)!=NULL;
-	return _iri_isgendelim(c);
+	return iri_isgendelim(c);
 }
 
 /**
@@ -173,7 +234,7 @@ bool wget_iri_isgendelim(char c)
 bool wget_iri_issubdelim(char c)
 {
 	// return strchr("!$&\'()*+,;=",c)!=NULL;
-	return _iri_issubdelim(c);
+	return iri_issubdelim(c);
 }
 
 /**
@@ -205,26 +266,15 @@ bool wget_iri_isreserved(char c)
  */
 bool wget_iri_isunreserved(char c)
 {
-	return c > 32 && c < 127 && (c_isalnum(c) || _iri_isunreserved(c));
+	return iri_isunreserved(c);
 }
 
-/**
- * \param[in] c A character
- * \return 1 if \p c is an unreserved character or a path separator, 0 if not
- *
- * Tests whether \p c is an unreserved character **or a path separator (`/`)**.
- */
-bool wget_iri_isunreserved_path(char c)
-{
-	return c > 32 && c < 127 && (c_isalnum(c) || _iri_isunreserved(c) || c == '/');
-}
-
-static unsigned char WGET_GCC_CONST _unhex(unsigned char c)
+static unsigned char WGET_GCC_CONST unhex(unsigned char c)
 {
 	return c <= '9' ? c - '0' : (c <= 'F' ? c - 'A' + 10 : c - 'a' + 10);
 }
 
-static char *_iri_unescape_inline(char *src, int ctype)
+static char *iri_unescape_inline(char *src, int ctype)
 {
 	char *ret = NULL;
 	unsigned char *s = (unsigned char *)src; // just a helper to avoid casting a lot
@@ -233,7 +283,7 @@ static char *_iri_unescape_inline(char *src, int ctype)
 	while (*s) {
 		if (*s == '%') {
 			if (c_isxdigit(s[1]) && c_isxdigit(s[2])) {
-				unsigned char c = (unsigned char) (_unhex(s[1]) << 4) | _unhex(s[2]);
+				unsigned char c = (unsigned char) (unhex(s[1]) << 4) | unhex(s[2]);
 				if (!ctype || (!(iri_ctype[(unsigned char)(c)] & ctype) && c != '%')) {
 					*d++ = c;
 					s += 3;
@@ -275,7 +325,7 @@ static char *_iri_unescape_inline(char *src, int ctype)
 			if (s[1] == 'x') {
 				unsigned char *p = s + 2;
 				while (c_isxdigit(*p)) {
-					value = (value << 4) | _unhex(*p);
+					value = (value << 4) | unhex(*p);
 					p++;
 				}
 				if (*p == ';') {
@@ -303,6 +353,10 @@ static char *_iri_unescape_inline(char *src, int ctype)
 					// this cannot be done inline since the URL's length may increase
 				}
 			}
+		} else if (*s == '\r' || *s == '\n') {
+			// Ignore / remove CR and LF from URLs. See https://gitlab.com/gnuwget/wget2/-/issues/522
+			s++;
+			continue;
 		}
 
 		*d++ = *s++;
@@ -324,7 +378,7 @@ static char *_iri_unescape_inline(char *src, int ctype)
  */
 char *wget_iri_unescape_inline(char *src)
 {
-	return _iri_unescape_inline(src, 0);
+	return iri_unescape_inline(src, 0);
 }
 
 /**
@@ -342,7 +396,7 @@ char *wget_iri_unescape_inline(char *src)
  */
 char *wget_iri_unescape_url_inline(char *src)
 {
-	return _iri_unescape_inline(src, IRI_CTYPE_GENDELIM);
+	return iri_unescape_inline(src, IRI_CTYPE_GENDELIM);
 }
 
 /**
@@ -452,7 +506,7 @@ wget_iri *wget_iri_parse(const char *url, const char *encoding)
 		const char *x;
 		have_scheme = 1;
 
-		for (x = url; *x && _iri_isscheme(*x); x++)
+		for (x = url; *x && iri_isscheme(*x); x++)
 			;
 
 		if (*x != ':' || c_isdigit(x[1]))
@@ -715,9 +769,10 @@ wget_iri *wget_iri_clone(const wget_iri *iri)
 
 /**
  * \param[in] iri An IRI
- * \return A string with the connection part of the IRI.
+ * \param[in] buf A buffer, where the resulting string will be put
+ * \return The contents of the buffer \p buf
  *
- * Return the connection part of the IRI \p iri.
+ * Append the connection part of the IRI \p iri to \p buf.
  *
  * The connection part is formed by the scheme, the hostname, and optionally the port. For example:
  *
@@ -727,26 +782,22 @@ wget_iri *wget_iri_clone(const wget_iri *iri)
  * It may be of the form `https://example.com:8080` if the port was provided when creating the IRI
  * or of the form `https://example.com` otherwise.
  */
-const char *wget_iri_get_connection_part(wget_iri *iri)
+const char *wget_iri_get_connection_part(const wget_iri *iri, wget_buffer *buf)
 {
 	if (iri) {
-		if (!iri->connection_part) {
-			if (iri->port_given) {
-				iri->connection_part =  wget_aprintf("%s://%s:%hu", schemes[iri->scheme].name, iri->host, iri->port);
-			} else {
-				iri->connection_part = wget_aprintf("%s://%s", schemes[iri->scheme].name, iri->host);
-			}
+		if (iri->port_given) {
+			wget_buffer_printf_append(buf, "%s://%s:%hu", schemes[iri->scheme].name, iri->host, iri->port);
+		} else {
+			wget_buffer_printf_append(buf, "%s://%s", schemes[iri->scheme].name, iri->host);
 		}
-
-		return iri->connection_part;
 	}
 
-	return NULL;
+	return buf->data;
 }
 
 // normalize /../ and remove /./
 
-static size_t WGET_GCC_NONNULL_ALL _normalize_path(char *path)
+static size_t WGET_GCC_NONNULL_ALL normalize_path(char *path)
 {
 	char *p1 = path, *p2 = path;
 
@@ -844,7 +895,7 @@ static size_t WGET_GCC_NONNULL_ALL _normalize_path(char *path)
  *
  * if \p len is `-1`, the length of \p val will be the result from `strlen(val)`.
  */
-const char *wget_iri_relative_to_abs(wget_iri *base, const char *val, size_t len, wget_buffer *buf)
+const char *wget_iri_relative_to_abs(const wget_iri *base, const char *val, size_t len, wget_buffer *buf)
 {
 	debug_printf("*url = %.*s\n", (int)len, val);
 
@@ -863,7 +914,7 @@ const char *wget_iri_relative_to_abs(wget_iri *base, const char *val, size_t len
 
 				// absolute URI without scheme: //authority/path...
 				if ((p = strchr(path + 2, '/')))
-					_normalize_path(p + 1);
+					normalize_path(p + 1);
 
 				wget_buffer_strcpy(buf, schemes[base->scheme].name);
 				wget_buffer_strcat(buf, ":");
@@ -871,9 +922,10 @@ const char *wget_iri_relative_to_abs(wget_iri *base, const char *val, size_t len
 				debug_printf("*1 %s\n", buf->data);
 			} else {
 				// absolute path
-				_normalize_path(path);
+				normalize_path(path);
 
-				wget_buffer_strcpy(buf, wget_iri_get_connection_part(base));
+				wget_buffer_reset(buf);
+				wget_iri_get_connection_part(base, buf);
 				wget_buffer_strcat(buf, "/");
 				wget_buffer_strcat(buf, path);
 				debug_printf("*2 %s\n", buf->data);
@@ -895,7 +947,8 @@ const char *wget_iri_relative_to_abs(wget_iri *base, const char *val, size_t len
 		} else if (base) {
 			// relative path
 			const char *lastsep = base->path ? strrchr(base->path, '/') : NULL;
-			wget_buffer_strcpy(buf, wget_iri_get_connection_part(base));
+			wget_buffer_reset(buf);
+			wget_iri_get_connection_part(base, buf);
 			wget_buffer_strcat(buf, "/");
 
 			size_t tmp_len = buf->length;
@@ -906,7 +959,7 @@ const char *wget_iri_relative_to_abs(wget_iri *base, const char *val, size_t len
 			if (len)
 				wget_buffer_memcat(buf, val, len);
 
-			buf->length = _normalize_path(buf->data + tmp_len) + tmp_len;
+			buf->length = normalize_path(buf->data + tmp_len) + tmp_len;
 
 			debug_printf("*4 %s %zu\n", buf->data, buf->length);
 		} else if (val[len] == 0) {
@@ -936,7 +989,7 @@ const char *wget_iri_relative_to_abs(wget_iri *base, const char *val, size_t len
  *
  * If \p base is NULL, then the parameter \p url must itself be an absolute URI.
  */
-wget_iri *wget_iri_parse_base(wget_iri *base, const char *url, const char *encoding)
+wget_iri *wget_iri_parse_base(const wget_iri *base, const char *url, const char *encoding)
 {
 	wget_iri *iri;
 
@@ -945,11 +998,11 @@ wget_iri *wget_iri_parse_base(wget_iri *base, const char *url, const char *encod
 		char sbuf[256];
 
 		wget_buffer_init(&buf, sbuf, sizeof(sbuf));
-		iri = wget_iri_parse(wget_iri_relative_to_abs(base, url, -1, &buf), encoding);
+		iri = wget_iri_parse(wget_iri_relative_to_abs(base, url, (size_t) -1, &buf), encoding);
 		wget_buffer_deinit(&buf);
 	} else {
 		// no base: just check URL for being an absolute URI
-		iri = wget_iri_parse(wget_iri_relative_to_abs(NULL, url, -1, NULL), encoding);
+		iri = wget_iri_parse(wget_iri_relative_to_abs(NULL, url, (size_t) -1, NULL), encoding);
 	}
 
 	return iri;
@@ -1033,7 +1086,7 @@ const char *wget_iri_escape(const char *src, wget_buffer *buf)
 		return buf->data;
 
 	for (begin = src; *src; src++) {
-		if (!wget_iri_isunreserved(*src)) {
+		if (!iri_isunreserved(*src)) {
 			if (begin != src)
 				wget_buffer_memcat(buf, begin, src - begin);
 			begin = src + 1;
@@ -1050,18 +1103,22 @@ const char *wget_iri_escape(const char *src, wget_buffer *buf)
 /**
  * \param[in] src A string, whose reserved characters are to be percent-encoded
  * \param[in] buf A buffer where the result will be copied.
- * \return The contents of the buffer \p buf after \p src has been encoded.
+ * \return The contents of the buffer \p buf after \p src has been encoded
+ * as described in https://datatracker.ietf.org/doc/html/rfc7230#section-5.3.1.
  *
- * Escapes (using percent-encoding) all the reserved characters in the string \p src
- * (just like wget_iri_escape()), **plus the path separator character `/`**. This function
- * is thus ideally suited for paths.
+ * Escapes the path part of the URI suitable for GET/POST requests (origin-form).
+ *   origin-form    = absolute-path [ "?" query ]
+ *   path-absolute = "/" [ segment-nz *( "/" segment ) ]
+ *   segment-nz    = 1*pchar
+ *   segment       = *pchar
+ *   pchar         = unreserved / pct-encoded / sub-delims / ":" / "@"
  */
 const char *wget_iri_escape_path(const char *src, wget_buffer *buf)
 {
 	const char *begin;
 
 	for (begin = src; *src; src++) {
-		if (!wget_iri_isunreserved_path(*src)) {
+		if (!(iri_isunreserved(*src) || iri_issubdelim(*src) || *src == '/' || *src == ':' || *src == '@')) {
 			if (begin != src)
 				wget_buffer_memcat(buf, begin, src - begin);
 			begin = src + 1;
@@ -1089,7 +1146,7 @@ const char *wget_iri_escape_query(const char *src, wget_buffer *buf)
 	const char *begin;
 
 	for (begin = src; *src; src++) {
-		if (!wget_iri_isunreserved(*src) && *src != '=' && *src != '&') {
+		if (!iri_isunreserved(*src) && *src != '=' && *src != '&') {
 			if (begin != src)
 				wget_buffer_memcat(buf, begin, src - begin);
 			begin = src + 1;
@@ -1127,6 +1184,11 @@ const char *wget_iri_get_escaped_host(const wget_iri *iri, wget_buffer *buf)
  * \return The contents of the buffer \p buf
  *
  * Return the resource string, suitable for use in HTTP requests.
+ * Details:
+ *   https://datatracker.ietf.org/doc/html/rfc7230#section-3.1.1
+ *   https://datatracker.ietf.org/doc/html/rfc7230#section-2.7
+ *   https://datatracker.ietf.org/doc/html/rfc3986#section-3.3
+ *
  * The resource string is comprised of the path, plus the query part, if present. Example:
  *
  *     /foo/bar/?param_1=one&param_2=two
@@ -1289,7 +1351,7 @@ char *wget_iri_get_query_as_filename(const wget_iri *iri, wget_buffer *buf, cons
  * If \p encoding is provided, this function will try to convert the path (which is originally
  * in UTF-8) to that encoding.
  */
-char *wget_iri_get_filename(const wget_iri *iri, wget_buffer *buf, const char *encoding)
+char *wget_iri_get_basename(const wget_iri *iri, wget_buffer *buf, const char *encoding, int flags)
 {
 	if (iri->path) {
 		char *fname;
@@ -1321,7 +1383,10 @@ char *wget_iri_get_filename(const wget_iri *iri, wget_buffer *buf, const char *e
 	if ((buf->length == 0 || buf->data[buf->length - 1] == '/') && default_page)
 		wget_buffer_memcat(buf, default_page, default_page_length);
 
-	return wget_iri_get_query_as_filename(iri, buf, encoding);
+	if (flags & WGET_IRI_WITH_QUERY)
+		return wget_iri_get_query_as_filename(iri, buf, encoding);
+
+	return buf->data;
 }
 
 // escaping: see https://tools.ietf.org/html/rfc2396#2 following (especially 2.4.2)
