@@ -1,6 +1,6 @@
 /*
- * Copyright(c) 2012 Tim Ruehsen
- * Copyright(c) 2015-2019 Free Software Foundation, Inc.
+ * Copyright (c) 2012 Tim Ruehsen
+ * Copyright (c) 2015-2021 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -107,7 +107,7 @@
  *   - Family: `AF_UNSPEC` (basically means "I don't care, pick the first one available").
  */
 
-static struct wget_tcp_st _global_tcp = {
+static struct wget_tcp_st global_tcp = {
 	.sockfd = -1,
 	.dns_timeout = -1,
 	.connect_timeout = -1,
@@ -123,15 +123,6 @@ static struct wget_tcp_st _global_tcp = {
 #endif
 };
 
-typedef struct
-{
-	const char
-		*hostname,
-		*ip;
-	uint16_t port;
-	long long dns_secs;	// milliseconds
-} _stats_data_t;
-
 /* for Windows compatibility */
 #include "sockets.h"
 /**
@@ -158,7 +149,7 @@ int wget_net_deinit(void)
 	return rc ? -1 : 0;
 }
 
-static int WGET_GCC_CONST _value_to_family(int value)
+static int WGET_GCC_CONST value_to_family(int value)
 {
 	switch (value) {
 	case WGET_NET_FAMILY_IPV4:
@@ -170,7 +161,7 @@ static int WGET_GCC_CONST _value_to_family(int value)
 	}
 }
 
-static int WGET_GCC_CONST _family_to_value(int family)
+static int WGET_GCC_CONST family_to_value(int family)
 {
 	switch (family) {
 	case AF_INET:
@@ -193,7 +184,7 @@ static int WGET_GCC_CONST _family_to_value(int family)
  */
 void wget_tcp_set_dns(wget_tcp *tcp, wget_dns *dns)
 {
-	(tcp ? tcp : &_global_tcp)->dns = dns;
+	(tcp ? tcp : &global_tcp)->dns = dns;
 }
 
 /**
@@ -206,10 +197,10 @@ void wget_tcp_set_dns(wget_tcp *tcp, wget_dns *dns)
  *
  * If \p tcp is NULL, TCP Fast Open is enabled or disabled globally.
  */
-void wget_tcp_set_tcp_fastopen(wget_tcp *tcp, int tcp_fastopen)
+void wget_tcp_set_tcp_fastopen(wget_tcp *tcp, bool tcp_fastopen)
 {
 #if defined TCP_FASTOPEN_OSX || defined TCP_FASTOPEN_LINUX || defined TCP_FASTOPEN_LINUX_411
-	(tcp ? tcp : &_global_tcp)->tcp_fastopen = !!tcp_fastopen;
+	(tcp ? tcp : &global_tcp)->tcp_fastopen = tcp_fastopen;
 #else
 	(void) tcp; (void) tcp_fastopen;
 #endif
@@ -223,9 +214,9 @@ void wget_tcp_set_tcp_fastopen(wget_tcp *tcp, int tcp_fastopen)
  *
  * You can enable and disable it with wget_tcp_set_tcp_fastopen().
  */
-char wget_tcp_get_tcp_fastopen(wget_tcp *tcp)
+bool wget_tcp_get_tcp_fastopen(wget_tcp *tcp)
 {
-	return (tcp ? tcp : &_global_tcp)->tcp_fastopen;
+	return (tcp ? tcp : &global_tcp)->tcp_fastopen;
 }
 
 /**
@@ -236,9 +227,9 @@ char wget_tcp_get_tcp_fastopen(wget_tcp *tcp)
  *
  * If \p tcp is NULL, TLS False Start is enabled or disabled globally.
  */
-void wget_tcp_set_tls_false_start(wget_tcp *tcp, int false_start)
+void wget_tcp_set_tls_false_start(wget_tcp *tcp, bool false_start)
 {
-	(tcp ? tcp : &_global_tcp)->tls_false_start = !!false_start;
+	(tcp ? tcp : &global_tcp)->tls_false_start = false_start;
 }
 
 /**
@@ -249,9 +240,9 @@ void wget_tcp_set_tls_false_start(wget_tcp *tcp, int false_start)
  *
  * You can enable and disable it with wget_tcp_set_tls_false_start().
  */
-char wget_tcp_get_tls_false_start(wget_tcp *tcp)
+bool wget_tcp_get_tls_false_start(wget_tcp *tcp)
 {
-	return (tcp ? tcp : &_global_tcp)->tls_false_start;
+	return (tcp ? tcp : &global_tcp)->tls_false_start;
 }
 
 /**
@@ -265,7 +256,7 @@ char wget_tcp_get_tls_false_start(wget_tcp *tcp)
  */
 void wget_tcp_set_protocol(wget_tcp *tcp, int protocol)
 {
-	(tcp ? tcp : &_global_tcp)->protocol = protocol;
+	(tcp ? tcp : &global_tcp)->protocol = protocol;
 }
 
 /**
@@ -276,7 +267,7 @@ void wget_tcp_set_protocol(wget_tcp *tcp, int protocol)
  */
 int wget_tcp_get_protocol(wget_tcp *tcp)
 {
-	return (tcp ? tcp : &_global_tcp)->protocol;
+	return (tcp ? tcp : &global_tcp)->protocol;
 }
 
 /**
@@ -291,7 +282,7 @@ int wget_tcp_get_protocol(wget_tcp *tcp)
  */
 void wget_tcp_set_preferred_family(wget_tcp *tcp, int family)
 {
-	(tcp ? tcp : &_global_tcp)->preferred_family = _value_to_family(family);
+	(tcp ? tcp : &global_tcp)->preferred_family = value_to_family(family);
 }
 
 /**
@@ -302,7 +293,7 @@ void wget_tcp_set_preferred_family(wget_tcp *tcp, int family)
  */
 int wget_tcp_get_preferred_family(wget_tcp *tcp)
 {
-	return _family_to_value((tcp ? tcp : &_global_tcp)->preferred_family);
+	return family_to_value((tcp ? tcp : &global_tcp)->preferred_family);
 }
 
 /**
@@ -317,7 +308,7 @@ int wget_tcp_get_preferred_family(wget_tcp *tcp)
  */
 void wget_tcp_set_family(wget_tcp *tcp, int family)
 {
-	(tcp ? tcp : &_global_tcp)->family = _value_to_family(family);
+	(tcp ? tcp : &global_tcp)->family = value_to_family(family);
 }
 
 /**
@@ -328,7 +319,7 @@ void wget_tcp_set_family(wget_tcp *tcp, int family)
  */
 int wget_tcp_get_family(wget_tcp *tcp)
 {
-	return _family_to_value((tcp ? tcp : &_global_tcp)->family);
+	return family_to_value((tcp ? tcp : &global_tcp)->family);
 }
 
 /**
@@ -372,7 +363,7 @@ int wget_tcp_get_local_port(wget_tcp *tcp)
  */
 void wget_tcp_set_connect_timeout(wget_tcp *tcp, int timeout)
 {
-	(tcp ? tcp : &_global_tcp)->connect_timeout = timeout;
+	(tcp ? tcp : &global_tcp)->connect_timeout = timeout;
 }
 
 /**
@@ -388,7 +379,7 @@ void wget_tcp_set_connect_timeout(wget_tcp *tcp, int timeout)
  */
 void wget_tcp_set_timeout(wget_tcp *tcp, int timeout)
 {
-	(tcp ? tcp : &_global_tcp)->timeout = timeout;
+	(tcp ? tcp : &global_tcp)->timeout = timeout;
 }
 
 /**
@@ -399,7 +390,7 @@ void wget_tcp_set_timeout(wget_tcp *tcp, int timeout)
  */
 int wget_tcp_get_timeout(wget_tcp *tcp)
 {
-	return (tcp ? tcp : &_global_tcp)->timeout;
+	return (tcp ? tcp : &global_tcp)->timeout;
 }
 
 /**
@@ -416,16 +407,12 @@ int wget_tcp_get_timeout(wget_tcp *tcp)
 void wget_tcp_set_bind_address(wget_tcp *tcp, const char *bind_address)
 {
 	if (!tcp)
-		tcp = &_global_tcp;
-
+		tcp = &global_tcp;
 
 	wget_dns_freeaddrinfo(tcp->dns, &tcp->bind_addrinfo);
 
 	if (bind_address) {
-		char copy[strlen(bind_address) + 1], *s = copy;
-		const char *host;
-
-		memcpy(copy, bind_address, sizeof(copy));
+		const char *host, *s = bind_address;
 
 		if (*s == '[') {
 			/* IPv6 address within brackets */
@@ -446,15 +433,35 @@ void wget_tcp_set_bind_address(wget_tcp *tcp, const char *bind_address)
 		}
 
 		if (*s == ':') {
-			*s++ = 0;
-			if (c_isdigit(*s))
-				tcp->bind_addrinfo = wget_dns_resolve(tcp->dns, host, (uint16_t) atoi(s), tcp->family, tcp->preferred_family);
+			char port[6];
+
+			wget_strscpy(port, s + 1, sizeof(port));
+
+			if (c_isdigit(*port))
+				tcp->bind_addrinfo = wget_dns_resolve(tcp->dns, host, (uint16_t) atoi(port), tcp->family, tcp->preferred_family);
 		} else {
 			tcp->bind_addrinfo = wget_dns_resolve(tcp->dns, host, 0, tcp->family, tcp->preferred_family);
 		}
 	}
 }
 
+/**
+ * \param[in] tcp A TCP connection. Might be NULL.
+ * \param[in] bind_interface A network interface name.
+ *
+ * Set the Network Interface the socket \p tcp will bind to on the local machine
+ * when connecting to a remote host.
+ *
+ * This is mainly relevant to wget_tcp_connect().
+ */
+void wget_tcp_set_bind_interface(wget_tcp *tcp, const char *bind_interface)
+{
+	if (!tcp)
+		tcp = &global_tcp;
+
+	tcp->bind_interface = bind_interface;
+}
+
 /**
  * \param[in] tcp A `wget_tcp` structure representing a TCP connection, returned by wget_tcp_init().
  * \param[in] ssl Flag to enable or disable SSL/TLS on the given connection.
@@ -463,9 +470,9 @@ void wget_tcp_set_bind_address(wget_tcp *tcp, const char *bind_address)
  *
  * If \p tcp is NULL, TLS will be enabled globally. Otherwise, TLS will be enabled only for the provided connection.
  */
-void wget_tcp_set_ssl(wget_tcp *tcp, int ssl)
+void wget_tcp_set_ssl(wget_tcp *tcp, bool ssl)
 {
-	(tcp ? tcp : &_global_tcp)->ssl = !!ssl;
+	(tcp ? tcp : &global_tcp)->ssl = ssl;
 }
 
 /**
@@ -474,9 +481,9 @@ void wget_tcp_set_ssl(wget_tcp *tcp, int ssl)
  *
  * Tells whether TLS is enabled or not.
  */
-int wget_tcp_get_ssl(wget_tcp *tcp)
+bool wget_tcp_get_ssl(wget_tcp *tcp)
 {
-	return (tcp ? tcp : &_global_tcp)->ssl;
+	return (tcp ? tcp : &global_tcp)->ssl;
 }
 
 /**
@@ -503,7 +510,7 @@ const char *wget_tcp_get_ip(wget_tcp *tcp)
 void wget_tcp_set_ssl_hostname(wget_tcp *tcp, const char *hostname)
 {
 	if (!tcp)
-		tcp = &_global_tcp;
+		tcp = &global_tcp;
 
 	xfree(tcp->ssl_hostname);
 	tcp->ssl_hostname = wget_strdup(hostname);
@@ -517,7 +524,7 @@ void wget_tcp_set_ssl_hostname(wget_tcp *tcp, const char *hostname)
  */
 const char *wget_tcp_get_ssl_hostname(wget_tcp *tcp)
 {
-	return (tcp ? tcp : &_global_tcp)->ssl_hostname;
+	return (tcp ? tcp : &global_tcp)->ssl_hostname;
 }
 
 /**
@@ -535,8 +542,8 @@ wget_tcp *wget_tcp_init(void)
 	wget_tcp *tcp = wget_malloc(sizeof(wget_tcp));
 
 	if (tcp) {
-		*tcp = _global_tcp;
-		tcp->ssl_hostname = wget_strdup(_global_tcp.ssl_hostname);
+		*tcp = global_tcp;
+		tcp->ssl_hostname = wget_strdup(global_tcp.ssl_hostname);
 	}
 
 	return tcp;
@@ -559,7 +566,7 @@ void wget_tcp_deinit(wget_tcp **_tcp)
 	wget_tcp *tcp;
 
 	if (!_tcp) {
-		xfree(_global_tcp.ssl_hostname);
+		xfree(global_tcp.ssl_hostname);
 		return;
 	}
 
@@ -594,7 +601,7 @@ static void _set_async(int fd)
 #endif
 }
 
-static void _set_socket_options(int fd)
+static void set_socket_options(const wget_tcp *tcp, int fd)
 {
 	int on = 1;
 
@@ -605,6 +612,17 @@ static void _set_socket_options(int fd)
 	if (setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, (void *)&on, sizeof(on)) == -1)
 		error_printf(_("Failed to set socket option NODELAY\n"));
 
+#ifdef SO_BINDTODEVICE
+	if (tcp->bind_interface) {
+		if (setsockopt(fd, SOL_SOCKET, SO_BINDTODEVICE, tcp->bind_interface, (socklen_t)strlen(tcp->bind_interface)) == -1)
+			error_printf(_("Failed to set socket option BINDTODEVICE\n"));
+	}
+#else
+	// Let's exit here instead of using a wrong interface (privacy concerns)
+	if (tcp->bind_interface)
+		error_printf_exit(_("Unsupported socket option BINDTODEVICE\n"));
+#endif
+
 #ifdef TCP_FASTOPEN_LINUX_411
 	on = 1;
 	if (setsockopt(fd, IPPROTO_TCP, TCP_FASTOPEN_CONNECT, (void *)&on, sizeof(on)) == -1)
@@ -644,6 +662,9 @@ int wget_tcp_ready_2_transfer(wget_tcp *tcp, int flags)
  * with wget_tcp_set_bind_address(). Otherwise the socket will bind to any address and port
  * chosen by the operating system.
  *
+ * You can also set which Network Interface on the local machine will the socket be bound to
+ * with wget_tcp_bind_interface().
+ *
  * This function will try to use TCP Fast Open if enabled and available. If TCP Fast Open fails,
  * it will fall back to the normal TCP handshake, without raising an error. You can enable TCP Fast Open
  * with wget_tcp_set_tcp_fastopen().
@@ -679,7 +700,7 @@ int wget_tcp_connect(wget_tcp *tcp, const char *host, uint16_t port)
 		int sockfd;
 		if ((sockfd = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol)) != -1) {
 			_set_async(sockfd);
-			_set_socket_options(sockfd);
+			set_socket_options(tcp, sockfd);
 
 			if (tcp->bind_addrinfo) {
 				if (debug) {
@@ -754,7 +775,7 @@ int wget_tcp_connect(wget_tcp *tcp, const char *host, uint16_t port)
 					}
 				}
 
-				if ((rc = getnameinfo(ai->ai_addr, ai->ai_addrlen, adr, sizeof(adr), s_port, sizeof(s_port), NI_NUMERICHOST | NI_NUMERICSERV)) == 0)
+				if (getnameinfo(ai->ai_addr, ai->ai_addrlen, adr, sizeof(adr), s_port, sizeof(s_port), NI_NUMERICHOST | NI_NUMERICSERV) == 0)
 					tcp->ip = wget_strdup(adr);
 				else
 					tcp->ip = NULL;
@@ -921,9 +942,9 @@ ssize_t wget_tcp_write(wget_tcp *tcp, const char *buf, size_t count)
 		} else {
 			if (errno != EAGAIN
 				&& errno != ENOTCONN
-				&& errno != EINPROGRESS
-			) {
-				error_printf(_("Failed to write %zu bytes (%d: %s)\n"), count, errno, strerror(errno));
+				&& errno != EINPROGRESS)
+			{
+				error_printf(_("Failed to write %zu bytes (%d)\n"), count, errno);
 				return -1;
 			}
 
