@@ -1,6 +1,6 @@
 /*
- * Copyright(c) 2012 Tim Ruehsen
- * Copyright(c) 2015-2016 Free Software Foundation, Inc.
+ * Copyright (c) 2012 Tim Ruehsen
+ * Copyright (c) 2015-2021 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -59,9 +59,9 @@ typedef struct {
 		token_len; // used bytes of token buffer (not counting terminating 0 byte)
 	void
 		*user_ctx; // user context (not needed if we were using nested functions)
-	wget_xml_callback_t
-		callback;
-} _xml_context;
+	wget_xml_callback
+		*callback;
+} xml_context;
 
 /* \cond _hide_internal_symbols */
 #define ascii_isspace(c) (c == ' ' || (c >= 9 && c <=  13))
@@ -72,7 +72,7 @@ typedef struct {
 
 // append a char to token buffer
 
-static const char *getToken(_xml_context *context)
+static const char *getToken(xml_context *context)
 {
 	int c;
 	const char *p;
@@ -80,7 +80,7 @@ static const char *getToken(_xml_context *context)
 	// skip leading whitespace
 	while ((c = *context->p) && ascii_isspace(c))
 		context->p++;
-	if (!c) return NULL;
+	if (!c) return NULL; // eof
 	context->token = context->p++;
 
 //	info_printf("a c=%c\n", c);
@@ -88,13 +88,14 @@ static const char *getToken(_xml_context *context)
 	if (ascii_isalpha(c) || c == '_') {
 		while ((c = *context->p) && !ascii_isspace(c) && c != '>' && c != '=')
 			context->p++;
-		if (!c) return NULL;
+		if (!c) return NULL; // syntax error
+
 		context->token_len = context->p - context->token;
 		return context->token;
 	}
 
 	if (c == '/') {
-		if (!(c = *context->p)) return NULL;
+		if (!(c = *context->p)) return NULL; // syntax error
 		context->p++;
 		if (c == '>') {
 			context->token_len = 2;
@@ -116,7 +117,7 @@ static const char *getToken(_xml_context *context)
 	}
 
 	if (c == '<') { // fetch specials, e.g. start of comments '<!--'
-		if (!(c = *context->p)) return NULL;
+		if (!(c = *context->p)) return NULL; // syntax error
 		context->p++;
 		if (c == '?' || c == '/') {
 			context->token_len = 2;
@@ -125,10 +126,10 @@ static const char *getToken(_xml_context *context)
 
 		if (c == '!') {
 			// left: <!--, <![CDATA[ and <!WHATEVER
-			if (!(c = *context->p)) return NULL;
+			if (!(c = *context->p)) return NULL; // syntax error
 			if (c == '-') {
 				context->p++;
-				if (!(c = *context->p)) return NULL;
+				if (!(c = *context->p)) return NULL; // syntax error
 				context->p++;
 				if (c == '-') {
 					context->token_len = 4;
@@ -155,12 +156,12 @@ static const char *getToken(_xml_context *context)
 	}
 
 	if (c == '-') { // fetch specials, e.g. end of comments '-->'
-		if (!(c = *context->p)) return NULL;
+		if (!(c = *context->p)) return NULL; // syntax error
 		if (c != '-') {
-			c = '-';
+			c = '-';  //???
 		} else {
 			context->p++;
-			if (!(c = *context->p)) return NULL;
+			if (!(c = *context->p)) return NULL; // syntax error
 			context->p++;
 			if (c != '>') {
 				context->p -= 2;
@@ -173,7 +174,7 @@ static const char *getToken(_xml_context *context)
 	}
 
 	if (c == '?') { // fetch specials, e.g. '?>'
-		if (!(c = *context->p)) return NULL;
+		if (!(c = *context->p)) return NULL; // syntax error
 		if (c != '>') {
 			// c = '?';
 		} else {
@@ -194,7 +195,7 @@ static const char *getToken(_xml_context *context)
 	return NULL;
 }
 
-static int getValue(_xml_context *context)
+static int getValue(xml_context *context)
 {
 	int c;
 
@@ -209,7 +210,7 @@ static int getValue(_xml_context *context)
 	if (c == '=') {
 		context->p++;
 		if (!getToken(context))
-			return EOF;
+			return EOF; // syntax error
 		else
 			return 1; // token valid
 	}
@@ -221,9 +222,9 @@ static int getValue(_xml_context *context)
 
 // special HTML <script> content parsing
 // see https://html.spec.whatwg.org/multipage/scripting.html#the-script-element
-// 4.3.1.2 Restrictions for contents of script elements
+// see https://html.spec.whatwg.org/multipage/scripting.html#restrictions-for-contents-of-script-elements
 
-static const char *getScriptContent(_xml_context *context)
+static const char *getScriptContent(xml_context *context)
 {
 	int comment = 0, length_valid = 0;
 	const char *p;
@@ -264,7 +265,50 @@ static const char *getScriptContent(_xml_context *context)
 	return context->token;
 }
 
-static const char *getUnparsed(_xml_context *context, int flags, const char *end, size_t len, const char *directory)
+// special HTML <style> content parsing
+// see https://html.spec.whatwg.org/multipage/semantics.html#the-style-element
+static const char *getStyleContent(xml_context *context)
+{
+	int comment = 0, length_valid = 0;
+	const char *p;
+
+	for (p = context->token = context->p; *p; p++) {
+		if (comment) {
+			if (p[0] == '*' && p[1] == '/') {
+				p += 2 - 1;
+				comment = 0;
+			}
+		} else {
+			if (p[0] == '/' && p[1] == '*') {
+				p += 2 - 1;
+				comment = 1;
+			} else if (*p == '<' && !wget_strncasecmp_ascii(p, "</style", 7)) {
+				context->token_len = p - context->token;
+				length_valid = 1;
+				for (p += 7; ascii_isspace(*p); p++);
+				if (*p == '>') {
+					p++;
+					break; // found end of <style>
+				} else if (!*p)
+					break; // end of input
+			}
+		}
+	}
+	context->p = p;
+
+	if (!length_valid)
+		context->token_len = p - context->token;
+
+	if (!*p && !context->token_len)
+		return NULL;
+
+	if (context->callback)
+		context->callback(context->user_ctx, XML_FLG_CONTENT | XML_FLG_END, "style", NULL, context->token, context->token_len, context->token - context->buf);
+
+	return context->token;
+}
+
+static const char *getUnparsed(xml_context *context, int flags, const char *end, size_t len, const char *directory)
 {
 	int c;
 
@@ -313,26 +357,26 @@ static const char *getUnparsed(_xml_context *context, int flags, const char *end
 	return context->token;
 }
 
-static const char *getComment(_xml_context *context)
+static const char *getComment(xml_context *context)
 {
 	return getUnparsed(context, XML_FLG_COMMENT, "-->", 3, NULL);
 }
 
-static const char *getProcessing(_xml_context *context)
+static const char *getProcessing(xml_context *context)
 {
 	return getUnparsed(context, XML_FLG_PROCESSING, "?>", 2, NULL);
 }
 
-static const char *getSpecial(_xml_context *context)
+static const char *getSpecial(xml_context *context)
 {
 	return getUnparsed(context, XML_FLG_SPECIAL, ">", 1, NULL);
 }
 
-static const char *getContent(_xml_context *context, const char *directory)
+static const char *getContent(xml_context *context, const char *directory)
 {
 	int c;
 
-		for (context->token = context->p; (c = *context->p) && c != '<'; context->p++);
+	for (context->token = context->p; (c = *context->p) && c != '<'; context->p++);
 
 	context->token_len = context->p - context->token;
 
@@ -346,7 +390,7 @@ static const char *getContent(_xml_context *context, const char *directory)
 	return context->token;
 }
 
-static void parseXML(const char *dir, _xml_context *context)
+static int parseXML(const char *dir, xml_context *context)
 {
 	const char *tok;
 	char directory[256] = "";
@@ -362,23 +406,24 @@ static void parseXML(const char *dir, _xml_context *context)
 		if (context->token_len)
 			debug_printf("%s='%.*s'\n", directory, (int)context->token_len, context->token);
 
-		if (!(tok = getToken(context))) return;
+		if (!(tok = getToken(context))) return WGET_E_SUCCESS;  //eof
 		// debug_printf("A Token '%.*s' len=%zu tok='%s'\n", (int)context->token_len, context->token, context->token_len, tok);
 
 		if (context->token_len == 1 && *tok == '<') {
 			// get element name and add it to directory
 			int flags = XML_FLG_BEGIN;
 
-			if (!(tok = getToken(context))) return;
+			if (!(tok = getToken(context))) return WGET_E_XML_PARSE_ERR; // syntax error
+
 			// debug_printf("A2 Token '%.*s'\n", (int)context->token_len, context->token);
 
 			if (!(context->hints & XML_HINT_HTML)) {
 				if (!pos || directory[pos - 1] != '/')
-					snprintf(&directory[pos], sizeof(directory) - pos, "/%.*s", (int)context->token_len, tok);
+					wget_snprintf(&directory[pos], sizeof(directory) - pos, "/%.*s", (int)context->token_len, tok);
 				else
-					snprintf(&directory[pos], sizeof(directory) - pos, "%.*s", (int)context->token_len, tok);
+					wget_snprintf(&directory[pos], sizeof(directory) - pos, "%.*s", (int)context->token_len, tok);
 			} else {
-				// snprintf(directory, sizeof(directory), "%.*s", (int)context->token_len, tok);
+				// wget_snprintf(directory, sizeof(directory), "%.*s", (int)context->token_len, tok);
 				if (context->token_len < sizeof(directory)) {
 					memcpy(directory, tok, context->token_len);
 					directory[context->token_len] = 0;
@@ -408,7 +453,7 @@ static void parseXML(const char *dir, _xml_context *context)
 								debug_printf("%s=%.*s\n", directory, (int)context->token_len, context->token);
 						}
 						else if (!wget_strcasecmp_ascii(directory, "style")) {
-							getContent(context, "style");
+							getStyleContent(context);
 							if (context->token_len)
 								debug_printf("%s=%.*s\n", directory, (int)context->token_len, context->token);
 						}
@@ -416,12 +461,14 @@ static void parseXML(const char *dir, _xml_context *context)
 						parseXML(directory, context); // descend one level
 					break;
 				} else {
-//					snprintf(attribute, sizeof(attribute), "%.*s", (int)context->token_len, tok);
-					char attribute[context->token_len + 1];
-					memcpy(attribute, tok, context->token_len);
-					attribute[context->token_len] = 0;
+					char attribute[256];
+					size_t attrlen = context->token_len >= sizeof(attribute) ? sizeof(attribute) - 1 : context->token_len;
+
+					memcpy(attribute, tok, attrlen);
+					attribute[attrlen] = 0;
+
+					if (getValue(context) == EOF) return WGET_E_XML_PARSE_ERR; // syntax error
 
-					if (getValue(context) == 0) return;
 					if (context->token_len) {
 						debug_printf("%s/@%s=%.*s\n", directory, attribute, (int)context->token_len, context->token);
 						if (context->callback)
@@ -439,7 +486,7 @@ static void parseXML(const char *dir, _xml_context *context)
 			if (!strncmp(tok, "</", 2)) {
 				// ascend one level
 				// cleanup - get name and '>'
-				if (!(tok = getToken(context))) return;
+				if (!(tok = getToken(context))) return WGET_E_XML_PARSE_ERR;
 				// debug_printf("X Token %s\n",tok);
 				if (context->callback) {
 					if (!(context->hints & XML_HINT_HTML))
@@ -451,10 +498,10 @@ static void parseXML(const char *dir, _xml_context *context)
 						context->callback(context->user_ctx, XML_FLG_END, tag, NULL, NULL, 0, 0);
 					}
 				}
-				if (!(tok = getToken(context))) return;
+				if (!(tok = getToken(context))) return WGET_E_XML_PARSE_ERR;
 				// debug_printf("Y Token %s\n",tok);
 				if (!(context->hints & XML_HINT_HTML))
-					return;
+					return WGET_E_SUCCESS;
 				else
 					continue;
 			} else if (!strncmp(tok, "<?", 2)) { // special info - ignore
@@ -471,6 +518,7 @@ static void parseXML(const char *dir, _xml_context *context)
 			continue;
 		}
 	} while (tok);
+	return WGET_E_SUCCESS;
 }
 
 /**
@@ -496,13 +544,13 @@ static void parseXML(const char *dir, _xml_context *context)
  *
  * %XML_HINT_HTML turns on HTML scanning.
  */
-void wget_xml_parse_buffer(
+int wget_xml_parse_buffer(
 	const char *buf,
-	wget_xml_callback_t callback,
+	wget_xml_callback *callback,
 	void *user_ctx,
 	int hints)
 {
-	_xml_context context;
+	xml_context context;
 
 	context.token = NULL;
 	context.token_size = 0;
@@ -513,7 +561,7 @@ void wget_xml_parse_buffer(
 	context.callback = callback;
 	context.hints = hints;
 
-	parseXML("/", &context);
+	return parseXML ("/", &context);
 }
 
 /**
@@ -526,7 +574,7 @@ void wget_xml_parse_buffer(
  */
 void wget_html_parse_buffer(
 	const char *buf,
-	wget_xml_callback_t callback,
+	wget_xml_callback *callback,
 	void *user_ctx,
 	int hints)
 {
@@ -545,7 +593,7 @@ void wget_html_parse_buffer(
  */
 void wget_xml_parse_file(
 	const char *fname,
-	wget_xml_callback_t callback,
+	wget_xml_callback *callback,
 	void *user_ctx,
 	int hints)
 {
@@ -559,7 +607,7 @@ void wget_xml_parse_file(
 				size_t nread = st.st_size;
 				char *buf = mmap(NULL, nread + 1, PROT_READ|PROT_WRITE, MAP_PRIVATE, fd, 0);
 #else
-				char *buf=xmalloc(st.st_size + 1);
+				char *buf=wget_malloc(st.st_size + 1);
 				size_t nread=read(fd, buf, st.st_size);
 #endif
 
@@ -582,16 +630,18 @@ void wget_xml_parse_file(
 		// maybe should use yy_scan_bytes instead of buffering into memory.
 		char tmp[4096];
 		ssize_t nbytes;
-		wget_buffer_t *buf = wget_buffer_alloc(4096);
+		wget_buffer buf;
+
+		wget_buffer_init(&buf, NULL, 4096);
 
 		while ((nbytes = read(STDIN_FILENO, tmp, sizeof(tmp))) > 0) {
-			wget_buffer_memcat(buf, tmp, nbytes);
+			wget_buffer_memcat(&buf, tmp, nbytes);
 		}
 
-		if (buf->length)
-			wget_xml_parse_buffer(buf->data, callback, user_ctx, hints);
+		if (buf.length)
+			wget_xml_parse_buffer(buf.data, callback, user_ctx, hints);
 
-		wget_buffer_free(&buf);
+		wget_buffer_deinit(&buf);
 	}
 }
 
@@ -607,7 +657,7 @@ void wget_xml_parse_file(
  */
 void wget_html_parse_file(
 	const char *fname,
-	wget_xml_callback_t callback,
+	wget_xml_callback *callback,
 	void *user_ctx,
 	int hints)
 {
