@@ -1,20 +1,20 @@
 /*
- * Copyright(c) 2017-2019 Free Software Foundation, Inc.
+ * Copyright (c) 2017-2021 Free Software Foundation, Inc.
  *
- * This file is part of libwget.
+ * This file is part of Wget
  *
- * Libwget is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published by
+ * Wget is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
- * Libwget is distributed in the hope that it will be useful,
+ * Wget is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with libwget.  If not, see <https://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with Wget  If not, see <https://www.gnu.org/licenses/>.
  *
  *
  * Testing Wget plugin behavior with non-existent plugins
@@ -174,7 +174,7 @@ int main(void)
 	if (access(".libs/libpluginname.so", R_OK) != 0
 		&& access(".libs/libpluginname.dll", R_OK) != 0
 		&& access(".libs/cygpluginname.dll", R_OK) != 0)
-		exit(77); // likely a static build
+		exit(WGET_TEST_EXIT_SKIP); // likely a static build
 
 	wget_test_start_server(
 			WGET_TEST_RESPONSE_URLS, &urls, countof(urls),
@@ -227,6 +227,6 @@ int main(void)
 			{	NULL } },
 		0);
 
-	exit(0);
+	exit(EXIT_SUCCESS);
 
 }
