@@ -1,21 +1,21 @@
 /*
- * Copyright(c) 2013 Tim Ruehsen
- * Copyright(c) 2015-2016 Free Software Foundation, Inc.
+ * Copyright (c) 2013 Tim Ruehsen
+ * Copyright (c) 2015-2021 Free Software Foundation, Inc.
  *
- * This file is part of libwget.
+ * This file is part of Wget
  *
- * Libwget is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published by
+ * Wget is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
- * Libwget is distributed in the hope that it will be useful,
+ * Wget is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with libwget.  If not, see <https://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with Wget  If not, see <https://www.gnu.org/licenses/>.
  *
  *
  * Testing Wget
@@ -298,126 +298,6 @@ int main(void)
 			{	NULL } },
 		0);
 
-	urls[3].headers[1] = "Last-Modified: Sat, 09 Oct 2004 08:30:00 GMT";
-
-	// test-N--no-content-disposition-trivial
-	wget_test(
-		WGET_TEST_OPTIONS, "-N --no-content-disposition",
-		WGET_TEST_REQUEST_URL, "dummy.txt",
-		WGET_TEST_EXPECTED_ERROR_CODE, 0,
-		WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
-			{	"dummy.txt", urls[3].body, 1097310600 },
-			{	NULL } },
-		0);
-
-	urls[3].headers[2] = "Content-Disposition: attachment; filename=\"filename.txt\"";
-
-	// test-N--no-content-disposition
-	wget_test(
-		WGET_TEST_OPTIONS, "-N --no-content-disposition",
-		WGET_TEST_REQUEST_URL, "dummy.txt",
-		WGET_TEST_EXPECTED_ERROR_CODE, 0,
-		WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
-			{	"dummy.txt", urls[3].body, 1097310600 },
-			{	NULL } },
-		0);
-
-	// test-N-HTTP--content-disposition
-	wget_test(
-		WGET_TEST_OPTIONS, "-N --content-disposition",
-		WGET_TEST_REQUEST_URL, "dummy.txt",
-		WGET_TEST_EXPECTED_ERROR_CODE, 0,
-		WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
-			{	"filename.txt", urls[3].body, 1097310600 },
-			{	NULL } },
-		0);
-
-	urls[3].headers[2] = NULL;
-
-	{
-		// server sends same length content with slightly different content
-		char modified[strlen(urls[3].body) + 1];
-
-		memcpy(modified, urls[3].body, sizeof(modified));
-		modified[3] = 'x';
-
-		urls[3].modified = 1097310600;
-
-		// test-N-current
-		wget_test(
-			WGET_TEST_OPTIONS, "-N",
-			WGET_TEST_REQUEST_URL, "dummy.txt",
-			WGET_TEST_EXPECTED_ERROR_CODE, 0,
-			WGET_TEST_EXISTING_FILES, &(wget_test_file_t []) {
-				{	"dummy.txt", modified, 1097310600 },
-				{	NULL } },
-			WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
-				{	"dummy.txt", modified, 1097310600 },
-				{	NULL } },
-			0);
-
-		// test-N-old
-		wget_test(
-			WGET_TEST_OPTIONS, "-N",
-			WGET_TEST_REQUEST_URL, "dummy.txt",
-			WGET_TEST_EXPECTED_ERROR_CODE, 0,
-			WGET_TEST_EXISTING_FILES, &(wget_test_file_t []) {
-				{	"dummy.txt", modified, 1097310000 }, // earlier timestamp
-				{	NULL } },
-			WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
-				{	"dummy.txt", urls[3].body, 1097310600 },
-				{	NULL } },
-			0);
-
-/*
-		// test-N-smaller
-		// This test just works with a HEAD request. But Wget uses If-Modified-Since.
-		const char *old_body = urls[3].body;
-		modified[strlen(modified)-2] = 0;
-		urls[3].body = modified;
-		wget_test(
-			WGET_TEST_OPTIONS, "-N",
-			WGET_TEST_REQUEST_URL, "dummy.txt",
-			WGET_TEST_EXPECTED_ERROR_CODE, 0,
-			WGET_TEST_EXISTING_FILES, &(wget_test_file_t []) {
-				{	"dummy.txt", old_body, 1097310600 },
-				{	NULL } },
-			WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
-				{	"dummy.txt", modified, 1097310600 },
-				{	NULL } },
-			0);
-		urls[3].body = old_body; // restore body
-*/
-	}
-
-	// test-N
-	wget_test(
-		WGET_TEST_OPTIONS, "-N",
-		WGET_TEST_REQUEST_URL, "dummy.txt",
-		WGET_TEST_EXPECTED_ERROR_CODE, 0,
-		WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
-			{	"dummy.txt", urls[3].body, 0},
-			{	NULL } },
-		0);
-
-	urls[3].headers[1] = NULL;
-
-/*
-	// test-N-no-info
-	// This test just works with a HEAD request. But Wget uses If-Modified-Since.
-	wget_test(
-		WGET_TEST_OPTIONS, "-N",
-		WGET_TEST_REQUEST_URL, "dummy.txt",
-		WGET_TEST_EXPECTED_ERROR_CODE, 0,
-		WGET_TEST_EXISTING_FILES, &(wget_test_file_t []) {
-			{	"dummy.txt", "anycontent", 1097310600 },
-			{	NULL } },
-		WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
-			{	"dummy.txt", urls[3].body, 0},
-			{	NULL } },
-		0);
-*/
-
 	urls[1].headers[1] = "Content-Disposition: attachment; filename=\"filename.html\"";
 
 	// test-O--no-content-disposition
@@ -597,6 +477,32 @@ int main(void)
 			{	NULL } },
 		0);
 
+	// test -c with existing empty file
+	wget_test(
+		WGET_TEST_OPTIONS, "-c",
+		WGET_TEST_REQUEST_URL, "index.html",
+		WGET_TEST_EXISTING_FILES, &(wget_test_file_t []) {
+			{ urls[0].name + 1, "" },
+			{	NULL } },
+		WGET_TEST_EXPECTED_ERROR_CODE, 0,
+		WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
+			{ urls[0].name + 1, urls[0].body },
+			{	NULL } },
+		0);
+
+	// test -c with existing complete file
+	wget_test(
+		WGET_TEST_OPTIONS, "-c",
+		WGET_TEST_REQUEST_URL, "index.html",
+		WGET_TEST_EXISTING_FILES, &(wget_test_file_t []) {
+			{ urls[0].name + 1, urls[0].body },
+			{	NULL } },
+		WGET_TEST_EXPECTED_ERROR_CODE, 0,
+		WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
+			{ urls[0].name + 1, urls[0].body },
+			{	NULL } },
+		0);
+
 	// test --chunk-size, new file
 	wget_test(
 		WGET_TEST_OPTIONS, "--chunk-size=3",
@@ -608,6 +514,17 @@ int main(void)
 			{	NULL } },
 		0);
 
+	// test --chunk-size with --progress, new file
+	wget_test(
+		WGET_TEST_OPTIONS, "--chunk-size=3 --progress=bar",
+		WGET_TEST_REQUEST_URL, "dummy.txt",
+		WGET_TEST_EXPECTED_ERROR_CODE, 0,
+		// no existing file
+		WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
+			{	"dummy.txt", urls[3].body },
+			{	NULL } },
+		0);
+
 	// test --chunk-size, new file, without Content-Length header
 	wget_test(
 		WGET_TEST_OPTIONS, "--chunk-size=3",
@@ -620,6 +537,18 @@ int main(void)
 			{	NULL } },
 		0);
 
+	// test --chunk-size with --progress, new file, without Content-Length header
+	wget_test(
+		WGET_TEST_OPTIONS, "--chunk-size=3 --progress=bar",
+		WGET_TEST_REQUEST_URL, "dummy.txt",
+		WGET_TEST_SERVER_SEND_CONTENT_LENGTH, 0, // server does not send Content-Length
+		WGET_TEST_EXPECTED_ERROR_CODE, 0,
+		// no existing file
+		WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
+			{	"dummy.txt", urls[3].body },
+			{	NULL } },
+		0);
+
 	// test --chunk-size, new file, with chunk size > Content-Length
 	wget_test(
 		WGET_TEST_OPTIONS, "--chunk-size=1000",
@@ -631,6 +560,17 @@ int main(void)
 			{	NULL } },
 		0);
 
+	// test --chunk-size with --progress, new file, with chunk size > Content-Length
+	wget_test(
+		WGET_TEST_OPTIONS, "--chunk-size=1000 --progress=bar",
+		WGET_TEST_REQUEST_URL, "dummy.txt",
+		WGET_TEST_EXPECTED_ERROR_CODE, 0,
+		// no existing file
+		WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
+			{	"dummy.txt", urls[3].body },
+			{	NULL } },
+		0);
+
 	// test -c --chunk-size, new file
 	wget_test(
 		WGET_TEST_OPTIONS, "--chunk-size=3 -c",
@@ -642,6 +582,17 @@ int main(void)
 			{	NULL } },
 		0);
 
+	// test -c --chunk-size with --progress, new file
+	wget_test(
+		WGET_TEST_OPTIONS, "--chunk-size=3 -c --progress=bar",
+		WGET_TEST_REQUEST_URL, "dummy.txt",
+		WGET_TEST_EXPECTED_ERROR_CODE, 0,
+		// no existing file
+		WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
+			{	"dummy.txt", urls[3].body },
+			{	NULL } },
+		0);
+
 	// test -c --chunk-size, existing file
 	wget_test(
 		WGET_TEST_OPTIONS, "--chunk-size=3 -c",
@@ -655,6 +606,19 @@ int main(void)
 			{	NULL } },
 		0);
 
+	// test -c --chunk-size with --progress, existing file
+	wget_test(
+		WGET_TEST_OPTIONS, "--chunk-size=3 -c --progress=bar",
+		WGET_TEST_REQUEST_URL, "dummy.txt",
+		WGET_TEST_EXPECTED_ERROR_CODE, 0,
+		WGET_TEST_EXISTING_FILES, &(wget_test_file_t []) {
+			{ "dummy.txt",  urls[3].body },
+			{	NULL } },
+		WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
+			{	"dummy.txt", urls[3].body },
+			{	NULL } },
+		0);
+
 	// test--https-only
 	wget_test(
 		WGET_TEST_OPTIONS, "--https-only -r -nH",
@@ -684,5 +648,5 @@ int main(void)
 			{	NULL } },
 		0);
 
-	exit(0);
+	exit(EXIT_SUCCESS);
 }
